"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GcsBucketPreparer = void 0;
const uuid_1 = require("uuid");
const storage_1 = require("@google-cloud/storage");
const BUCKET_PREFIX = "reg-publish-bucket";
class GcsBucketPreparer {
    inquire() {
        return [
            // {
            //   name: "projectId",
            //   type: "input",
            //   message: "Which GCP project id",
            // },
            {
                name: "createBucket",
                type: "confirm",
                message: "Create a new GCS bucket",
                default: true,
            },
            {
                name: "bucketName",
                type: "input",
                message: "Existing bucket name",
                when: (ctx) => !ctx.createBucket,
            },
        ];
    }
    prepare(config) {
        this._logger = config.logger;
        const ir = config.options;
        if (!ir.createBucket) {
            return Promise.resolve({
                // projectId: ir.projectId as string,
                bucketName: ir.bucketName,
            });
        }
        else {
            const id = (0, uuid_1.v4)();
            const bucketName = `${BUCKET_PREFIX}-${id}`;
            if (config.noEmit) {
                this._logger.info(`Skip to create GCS bucket ${bucketName} because noEmit option.`);
                return Promise.resolve({ bucketName });
            }
            this._logger.info(`Create new GCS bucket: ${this._logger.colors.magenta(bucketName)}`);
            const spinner = this._logger.getSpinner(`creating bucket...`);
            spinner.start();
            return this._createBucket(bucketName).then(bucket => {
                spinner.stop();
                return { bucketName: bucket.name };
            });
        }
    }
    _createBucket(bucketName) {
        return __awaiter(this, void 0, void 0, function* () {
            const bucket = new storage_1.Storage().bucket(bucketName);
            yield bucket.create({
                coldline: true,
            });
            yield bucket.acl.default.add({
                entity: "allUsers",
                role: "READER",
            });
            return bucket;
        });
    }
}
exports.GcsBucketPreparer = GcsBucketPreparer;
//# sourceMappingURL=gcs-bucket-preparer.js.map