"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GcsPublisherPlugin = void 0;
const path_1 = __importDefault(require("path"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const storage_1 = require("@google-cloud/storage");
const reg_suit_util_1 = require("reg-suit-util");
class GcsPublisherPlugin extends reg_suit_util_1.AbstractPublisher {
    constructor() {
        super();
        this.name = "reg-publish-gcs-plugin";
    }
    init(config) {
        this.noEmit = config.noEmit;
        this.logger = config.logger;
        this._options = config;
        this._pluginConfig = Object.assign({}, config.options);
        this._gcsClient = new storage_1.Storage();
    }
    publish(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const { indexFile } = yield this.publishInternal(key);
            const reportUrl = indexFile && `${this.getUriPrefix()}/${this.resolveInBucket(key)}/${indexFile.path}`;
            return { reportUrl };
        });
    }
    fetch(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.fetchInternal(key);
        });
    }
    getUriPrefix() {
        const { customUri } = this._pluginConfig;
        if (customUri) {
            return customUri.endsWith("/") ? customUri.slice(0, customUri.length - 1) : customUri;
        }
        else {
            return `https://storage.googleapis.com/${this.getBucketName()}`;
        }
    }
    getBucketRootDir() {
        return this._pluginConfig.pathPrefix;
    }
    getBucketName() {
        return this._pluginConfig.bucketName;
    }
    getLocalGlobPattern() {
        return this._pluginConfig.pattern;
    }
    getWorkingDirs() {
        return this._options.workingDirs;
    }
    uploadItem(key, item) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._gcsClient.bucket(this._pluginConfig.bucketName).upload(item.absPath, {
                destination: `${key}/${item.path}`,
                gzip: true,
            });
            this.logger.verbose(`Uploaded from ${item.absPath} to ${key}/${item.path}`);
            return item;
        });
    }
    downloadItem({ remotePath }, item) {
        return __awaiter(this, void 0, void 0, function* () {
            mkdirp_1.default.sync(path_1.default.dirname(item.absPath));
            yield this._gcsClient.bucket(this._pluginConfig.bucketName).file(remotePath).download({
                destination: item.absPath,
                validation: false,
            });
            this.logger.verbose(`Downloaded from ${remotePath} to ${item.absPath}`);
            return item;
        });
    }
    listItems(lastKey, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this._gcsClient.bucket(this._pluginConfig.bucketName).getFiles({
                    prefix,
                    maxResults: 1000,
                    pageToken: lastKey,
                }, (err, files, nextQuery) => {
                    if (err) {
                        reject(err);
                    }
                    const nextMarker = nextQuery && nextQuery.pageToken;
                    resolve({
                        isTruncated: nextMarker != null,
                        contents: !files ? [] : files.map(f => ({ key: f.name })),
                        nextMarker: nextMarker,
                    });
                });
            });
        });
    }
}
exports.GcsPublisherPlugin = GcsPublisherPlugin;
//# sourceMappingURL=gcs-publisher-plugin.js.map